#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

import os
import re


MANUAL_INDEX_PATH = "/usr/local/share/manual/package/index.html"
MANUAL_LIST_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                                    "manual_list.html")
MANUAL_BODY_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                                    "manual_body.html")

CONTAINER_ID_RE = 'id\s*=\s*"container"'
CONTENT_ID_RE = 'id\s*=\s*"content"'
INDEX_RE = "<!--\s*index\s*=\s*([0-9]+)\s*[^>]*>"


def _get_file_data(file_path):
    """Return data in "file_path"."""

    with open(file_path) as file_handle:
        return file_handle.read()


def _get_temp_path(man_path):
    """Return temporary path for "man_path"."""

    return man_path + "." + str(os.getpid())


def _get_index_no(data):
    """Return temporary path for "man_path"."""

    match = re.search(INDEX_RE, data)
    if not match:
        return 0

    return int(match.group(1))


def _insert_pkg_manual(man_data, pkg_man_data, start_re):
    """Insert "pkg_man_data" in "man_data"."""

    index_no = _get_index_no(pkg_man_data)

    start_match = re.search(start_re, man_data)
    if not start_match:
        return man_data

    idx = start_match.end(0)
    while True:
        match = re.search(INDEX_RE, man_data[idx:])
        if not match:
            return man_data

        try:
            if index_no < int(match.group(1)):
                insert_idx = idx + match.start(0)
                man_data = man_data[:insert_idx] + pkg_man_data + \
                                man_data[insert_idx:]
                return man_data

            idx += match.end(0)
        except:
            # Ignore error
            continue


def _remove_pkg_manual(man_data, pkg_man_data, start_re):
    """remove "pkg_man_data" in "man_data"."""

    index_no = _get_index_no(pkg_man_data)

    start_match = re.search(start_re, man_data)
    if not start_match:
        return man_data

    remove_start = 0
    idx = start_match.end(0)
    while True:
        match = re.search(INDEX_RE, man_data[idx:])
        if not match:
            return man_data

        try:
            if index_no == int(match.group(1)):
                remove_start = idx + match.start(0)
            elif remove_start and index_no < int(match.group(1)):
                remove_end = idx + match.start(0)
                man_data = man_data[:remove_start] + man_data[remove_end:]
                return man_data

            idx += match.end(0)
        except:
            # Ignore error
            continue


def install():
    """Install manual."""

    man_data = _get_file_data(MANUAL_INDEX_PATH)
    list_data = _get_file_data(MANUAL_LIST_PATH)
    body_data = _get_file_data(MANUAL_BODY_PATH)

    if not man_data:
        return

    if list_data:
        man_data = _insert_pkg_manual(man_data, list_data, CONTAINER_ID_RE)

    if body_data:
        man_data = _insert_pkg_manual(man_data, body_data, CONTENT_ID_RE)

    temp_path = _get_temp_path(MANUAL_INDEX_PATH)
    with open(temp_path, "w") as temp_file:
        temp_file.write(man_data)

    os.rename(temp_path, MANUAL_INDEX_PATH)


def uninstall():
    """uninstall manual."""

    man_data = _get_file_data(MANUAL_INDEX_PATH)
    list_data = _get_file_data(MANUAL_LIST_PATH)
    body_data = _get_file_data(MANUAL_BODY_PATH)

    if not man_data:
        return

    if list_data:
        man_data = _remove_pkg_manual(man_data, list_data, CONTAINER_ID_RE)

    if body_data:
        man_data = _remove_pkg_manual(man_data, body_data, CONTENT_ID_RE)

    temp_path = _get_temp_path(MANUAL_INDEX_PATH)
    with open(temp_path, "w") as temp_file:
        temp_file.write(man_data)

    os.rename(temp_path, MANUAL_INDEX_PATH)


def update():
    """Update manual."""

    uninstall()
    install()
